/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   http://www.tkiraaly.hu
*   Date         -  2013.01.31.
*   Chip         -  ATmega8, APH2
*   Compiler     -  avr-gcc ( WinAVR)
*   
*   A program 9600 Baud 8N1 mellett a berkezo karaktereket echo-zza (visszakuldi).  
*   Adas/vetel ellenorzesehez. 
*
********************************************************************************
*
*   CKSEL3, CKSEL2, CKSEL1, CKSEL0
*   P       P       P       P       0000  External Clock
*   
*   P       P       P       -       0001  Calibrated Internal RC Oscillator 1MHz
*   P       P       -       P       0010  Calibrated Internal RC Oscillator 2MHz
*   P       P       -       -       0011  Calibrated Internal RC Oscillator 4MHz
*   P       -       P       P       0100  Calibrated Internal RC Oscillator 8MHz
*
*   P       -       P       -       0101  External RC Oscillator 0.1-0.9MHz
*   P       -       -       P       0110  External RC Oscillator 0.9-3MHz
*   P       -       -       -       0111  External RC Oscillator 3-8MHz
*   -       P       P       P       1000  External RC Oscillator 8MHz-
*
*   -       P       P       -       1001  External Low-frequency Crystal
*
*   -       P       -       P       1010  Ceramic resonator 0.4-0.9MHz
*   -       P       -       -       1011  Crystal Oscillator 0.4-0.9MHz
*   -       -       P       P       1100  Ceramic resonator 0.9-3MHz
*   -       -       P       -       1101  Crystal Oscillator 0.9-3MHz
*   -       -       -       P       1110  Ceramic resonator 3-8MHz
*   -       -       -       P       1110  Ceramic resonator 8MHz-, +CKOPT P
*   -       -       -       -       1111  Crystal Oscillator 3-8MHz
*   -       -       -       -       1111  Crystal Oscillator 8MHz-, +CKOPT P
*
*   Calibrated Internal RC Oscillator
*   SUT1  SUT0
*   P     P         BOD enabled
*   P     -         Fast rising power
*   -     P         Slowly rising power
*
********************************************************************************
*
*   PB0 (14) - LED - 470R
*   TXD (2)
*   RXD (3)
*
*******************************************************************************/

#define F_CPU          4000000                   // orajel 4MHz
#define BAUDRATE       9600
#define MYUBRR         F_CPU/ 16UL/ BAUDRATE - 1




#include "tkiraaly_atmega8.h"
#include <util/delay.h>
#include <avr/pgmspace.h>




#define LED                  0
#define LED_ENABLE           BS( DDRB, LED)
#define LED_BE               BC( PORTB, LED)
#define LED_KI               BS( PORTB, LED)




// UCSRA beallitasai
#define DOUBLE_SPEED         0B00000010          // ketszeres sebesseg

// UCSRB beallitasai
#define IT_RX_ENABLE         0B10000000          // vetel kesz megszakitas engedelyezese
#define IT_TX_ENABLE         0B01000000          // adas kesz megszakitas engedelyezese
#define IT_UDR_EMPTY_ENABLE  0B00100000          // adat buffer ures megszakitas engedelyezese
#define RX_ENABLE            0B00010000          // vetel engedelyezese
#define TX_ENABLE            0B00001000          // adas engedelyezese

// UCSRC beallitasai
#define SET_UCSRC            0B10000000          // UCSRC kivalasztasa
#define SET_UBRRH            0                   // UBRRB kivalasztasa
#define ASYNCRON             0                   // asszinkron mukodes
#define SYNCRON              0B01000000          // szinkron mukodes
#define PARITY_NO            0                   // nincs paritas bit
#define PARITY_EVEN          0B00100000          // paros paritas
#define PARITY_ODD           0B00110000          // paratlan paritas
#define STOPBIT_1            0                   // 1 stopbit
#define STOPBIT_2            0B00001000          // 2 stopbit
#define BIT_5                0                   // 5 bit-es karakterhossz
#define BIT_6                0B00000010          // 6 bit-es karakterhossz
#define BIT_7                0B00000100          // 7 bit-es karakterhossz
#define BIT_8                0B00000110          // 8 bit-es karakterhossz




UC volatile usart_input= 0;                      // soros portrol bejovo betu




void usart_putc( UC);                            // USART egy karakter kuldese



 
ISR( USART_RXC_vect)                             // USART megszakitas kezelese
{
   usart_input= UDR;
}




int main( void)
{
   UBRRL= MYUBRR;                            // USART inicializalasa
   UBRRH= ( MYUBRR)>> 8;
   UCSRC= SET_UCSRC+ ASYNCRON+ BIT_8+ PARITY_NO+ STOPBIT_1;
   UCSRB= RX_ENABLE+ TX_ENABLE+ IT_RX_ENABLE;
   IT_ENABLE;
  
   LED_ENABLE;
   LED_BE;
   
   for(;;)
   {
      while( usart_input == 0 );
      usart_putc( usart_input);
      usart_input= 0;
   }
}




void usart_putc( UC c)                           // USART egy karakter kuldese
{
    while( BTC( UCSRA, UDRE));                   // 1, ha kesz az adat fogadasara
    UDR= c;
}
